@@ -92,12 +92,11 @@ def internal_link(
 ) -> str:
     if not config.create_internal_links:
         return text
-    else:
-        if title:
-            title = f' title="{title}"'
-        if css_class:
-            css_class = f' class="{css_class}"'
-        return f'<a{css_class} href="{href}"{title}>{text}</a>'
+    if title:
+        title = f' title="{title}"'
+    if css_class:
+        css_class = f' class="{css_class}"'
+    return f'<a{css_class} href="{href}"{title}>{text}</a>'
 
 
 def indent(n: int, content, single_indent: int = 2):
@@ -164,23 +163,29 @@ def bibtex_to_html(entry):
 
 def authors_to_html(entry, config: Config):
     try:
-        authors = get_authors(entry)
-        authors_html = (
-            _get_list_of_names(authors, css_class="author", config=config) + "."
-        )
-    except KeyError:
-        try:
+        if _put_editors_in_front(entry):
             authors = get_editors(entry)
             authors_html = (
                 _get_list_of_names(authors, css_class="author", config=config)
                 + ", editors"
                 + "."
             )
-        except KeyError:
-            authors_html = ""
-    if not authors_html:
+        else:
+            authors = get_authors(entry)
+            authors_html = (
+                _get_list_of_names(authors, css_class="author", config=config) + "."
+            )
+        return f'<span class="author-list">{authors_html}</span>'
+    except KeyError:
         return ""
-    return f'<span class="author-list">{authors_html}</span>'
+
+
+def _put_editors_in_front(entry):
+    return (
+        entry["ENTRYTYPE"] in ("book", "proceedings")
+        and "author" not in entry
+        and "editor" in entry
+    )
 
 
 def editors_to_html(entry, config: Config):
@@ -201,7 +206,7 @@ def funders_to_html(entry, config):
         funders = entry["funding"]
     except KeyError:
         return ""
-    html_items = list()
+    html_items = []
     for f in funders:
         html_items.append(
             internal_link(
@@ -225,7 +230,7 @@ def _get_list_of_names(
     delimiter_intermediate=",\n",
     delimiter_last=" and\n",
 ):
-    ls = list()
+    ls = []
     for a in names:
         html_a = _name_to_html(a)
         if config.index_author(a):
@@ -241,108 +246,146 @@ def _get_list_of_names(
     return delimiter_last.join(ls)
 
 
-def publication_venue_to_html(entry, config):
-    def get_day_html(entry):
+def get_date_html(entry):
+    def _get_day_html(entry):
         day = entry.get("day", None)
         if not day:
             return ""
         return f'<span class="day">{day}.</span>'
 
-    def get_month_html(entry):
+    def _get_month_html(entry):
         month = entry.get("month", None)
         if not month:
             return ""
         return f'<span class="month">{month}</span>'
 
-    def get_year_html(entry):
+    def _get_year_html(entry):
         year = get_year(entry)
         if not year:
             return ""
         return f'<span class="year">{year}</span>'
 
-    date_html = "\n".join(
+    return "\n".join(
         [
             e
-            for e in [get_day_html(entry), get_month_html(entry), get_year_html(entry)]
+            for e in [
+                _get_day_html(entry),
+                _get_month_html(entry),
+                _get_year_html(entry),
+            ]
             if e
         ]
     )
+
+
+def _get_techreport_html(entry):
+    if entry["ENTRYTYPE"] != "techreport" or "number" not in entry:
+        return None
+    return f'Technical report <span class="report-number">{entry["number"]}</span>'
+
+
+def _get_publishing_institution(entry):
+    try:
+        return f'<span class="publishing-institution">{entry["institution"]}</span>'
+    except KeyError:
+        return None
+
+
+def _get_explicit_venue(entry):
+    try:
+        return f'<span class="venue">at {entry["venue"]}</span>'
+    except KeyError:
+        return None
+
+
+def _get_journal_info_html(entry):
+    assert "journal" in entry
+    try:
+        pages = entry["pages"]
+    except KeyError:
+        pages = ""
+    if pages:
+        journal_pages = f'<span class="pages">:{pages}</span>'
+    else:
+        journal_pages = ""
+    journal = f"<em class=\"journaltitle\">{entry['journal']}</em>"
+    try:
+        volume = f", <span class=\"journalvolume\">{entry['volume']}</span>"
+    except KeyError:
+        volume = ""
+    try:
+        number = f"<span class=\"journalnumber\">({entry['number']})</span>"
+    except KeyError:
+        number = ""
+    return f'<span class="journal">{journal}{volume}{number}{journal_pages}</span>'
+
+
+def _get_pages_html(entry):
+    try:
+        return f'<span class="pages">pages {entry["pages"]}</span>'
+    except KeyError:
+        return None
+
+
+def _get_series_html(entry):
+    try:
+        return f"<span class=\"series\">{entry['series']}</span>"
+    except KeyError:
+        return None
+
+
+def _get_booktitle_html(entry):
+    try:
+        return f"<em class=\"booktitle\">{indent(1, entry['booktitle'])}</em>"
+    except KeyError:
+        logging.debug("No booktitle for %s", entry)
+        return None
+
+
+def publication_venue_to_html(entry, config):
+    date_html = get_date_html(entry)
     if date_html:
         date = f'<span class="publication-date">{indent(1, date_html)}</span>'
     else:
         date = ""
 
-    if entry["ENTRYTYPE"] == "techreport" and "number" in entry:
-        howpublished = (
-            f'Technical report <span class="report-number">{entry["number"]}</span>'
-        )
-    else:
-        howpublished = ""
-    try:
-        institution = (
-            f'<span class="publishing-institution">{entry["institution"]}</span>'
-        )
-    except KeyError:
-        institution = ""
-    try:
-        venue = f'<span class="venue">at {entry["venue"]}</span>'
-    except KeyError:
-        venue = ""
-
-    howpublished = ", ".join(
-        [
-            e
-            for e in (venue, entry.get("howpublished", None), howpublished, institution)
-            if e
-        ]
+    # this is information that does not conform to the commonly seen
+    # conference-, journal- and book-publications.
+    special_publication_info = (
+        _get_explicit_venue(entry),
+        entry.get("howpublished", None),
+        _get_techreport_html(entry),
+        _get_publishing_institution(entry),
     )
-    if howpublished:
-        howpublished = f'<span class="publicationmethod">{howpublished}</span>'
-    else:
-        howpublished = ""
 
-    try:
-        pages = entry["pages"]
-    except KeyError:
-        pages = ""
+    if any(special_publication_info):
+        publication_method = ", ".join(special_publication_info)
+        assert publication_method
+        publication_method = (
+            f'<span class="publicationmethod">{publication_method}</span>'
+        )
+    else:
+        publication_method = None
 
     if "journal" in entry:
-        if pages:
-            journal_pages = f'<span class="pages">:{pages}</span>'
-            pages = ""
-        else:
-            journal_pages = ""
-        journal = f"<em class=\"journaltitle\">{entry['journal']}</em>"
-        try:
-            volume = f", <span class=\"journalvolume\">{entry['volume']}</span>"
-        except KeyError:
-            volume = ""
-        try:
-            number = f"<span class=\"journalnumber\">({entry['number']})</span>"
-        except KeyError:
-            number = ""
-        venue = f'<span class="journal">{journal}{volume}{number}{journal_pages}</span>'
+        venue = _get_journal_info_html(entry)
     else:
-        if "author" in entry:
-            editors = editors_to_html(entry, config)
-        else:
+        if _put_editors_in_front(entry):
+            # If the editors are put in front of the bibtex entry
+            # like so:
+            # Dirk Beyer and Damien Zufferey, editors. Proceedings of [...]
+            # then we don't want to list the editors redundantly
+            # as part of the publication venue, so we leave the editors blank here.
             editors = ""
-        try:
-            booktitle = f"<em class=\"booktitle\">{indent(1, entry['booktitle'])}</em>"
-        except KeyError:
-            logging.debug("No booktitle for %s", entry)
-            booktitle = ""
-        if pages:
-            pages = f'<span class="pages">pages {pages}</span>'
-        try:
-            series = f"<span class=\"series\">{entry['series']}</span>"
-        except KeyError:
-            series = None
-        venue = ""
+        else:
+            editors = editors_to_html(entry, config)
+        booktitle = _get_booktitle_html(entry)
+        pages = _get_pages_html(entry)
+        series = _get_series_html(entry)
+        venue = ",\n".join([k for k in (editors, booktitle, series, pages) if k])
         if booktitle:
-            venue += "In "
-        venue += ",\n".join([k for k in (editors, booktitle, series, pages) if k])
-    all_infos = ",\n".join([e for e in (venue, howpublished, date) if e])
+            venue = f"In {venue}"
+    all_infos = ",\n".join([e for e in (venue, publication_method, date) if e])
     return f'<span class="howpublished">\n{indent(1, all_infos)}.\n</span>'
 
 
@@ -364,7 +407,7 @@ def artifacts_to_html(entry):
     artifact_keys = sorted([k for k in entry if re.match(r"artifact[0-9]*", k)])
     if not artifact_keys:
         return ""
-    html_lines = list()
+    html_lines = []
     for artifact in [entry[k] for k in artifact_keys]:
         html_lines.append(f'<a href="https://doi.org/{artifact}">doi:{artifact}</a>')
     assert len(html_lines) > 0
@@ -419,12 +462,17 @@ def publisher_version_to_html(entry):
     description = "Link to official version of paper (may be a landing page with more information)"
     try:
         doi = get_doi(entry)
-        return f'<a class="doi-link" href="https://doi.org/{doi}" title="{description}"><img alt="" height="17px" width="17px" src="https://www.sosy-lab.org/research/pub/Icons/doi.svg">Publisher\'s Version</a>'
+        return f'<a class="publishers-version-link doi-link" href="https://doi.org/{doi}" title="{description}"><img alt="" height="17px" width="17px" src="https://www.sosy-lab.org/research/pub/Icons/doi.svg">Publisher\'s Version</a>'
     except KeyError:
         pass
     try:
         urn = get_urn(entry)
-        return f'<a class="urn-link" href="https://nbn-resolving.org/process-urn-form?identifier={urn}&verb=REDIRECT" title="{description}">Publisher\'s Version</a>'
+        return f'<a class="publishers-version-link urn-link" href="https://nbn-resolving.org/process-urn-form?identifier={urn}&verb=REDIRECT" title="{description}">Publisher\'s Version</a>'
+    except KeyError:
+        pass
+    try:
+        publisher_url = get_publishers_version_url(entry)
+        return f'<a class="publishers-version-link" href="{publisher_url}" title="{description}">Publisher\'s Version</a>'
     except KeyError:
         return ""
 
@@ -478,6 +526,13 @@ def get_urn(entry):
     return urn
 
 
+def get_publishers_version_url(entry):
+    url = entry["urlpub"]
+    if not url:
+        raise KeyError
+    return url
+
+
 def html_id(entry):
     return entry["ID"].replace(" ", "")
 
@@ -523,7 +578,7 @@ def write(
 ):
     output_file = Path(output_file)
     output_file.parent.mkdir(exist_ok=True, parents=True)
-    with open(output_file, "w") as outp:
+    with open(output_file, "w", encoding="UTF-8") as outp:
         html = writes(entries, config=config, head=head, header_title=header_title)
         write_html(outp, html)
 
@@ -663,7 +718,7 @@ def write_html_tree(entries: Sequence[dict], output_root, *, config, header_titl
                 yield f'<li><a href="{link}">{name}</a></li>'
             yield "</ul>"
 
-    with open(output_root / "index.html", "w") as outp:
+    with open(output_root / "index.html", "w", encoding="UTF-8") as outp:
         write_html(outp, "\n".join(index_page(header_title)))
 
 
@@ -691,7 +746,7 @@ def write_year_tree(entries: Sequence[dict], output_root, *, config, header_titl
 
 
 def _unite_same_authors_long_and_shortform(by_author):
-    author_pages = dict()
+    author_pages = {}
     Author = namedtuple("Author", ["name", "link", "entries"])
     for author, es in by_author.items():
         link = _author_to_link(author.name)
